#!/bin/bash

# Function to display usage
usage() {
    echo "Usage: $0 --usrp=<serial_number> --gnb | --ue"
    echo
    echo "Options:"
    echo "  --gnb                   Start gNB"
    echo "  --help                  Display this help message"
    exit 1
}

# Function to parse the command-line arguments
parse_args() {
    if [[ $# -lt 1 ]]; then
        usage
    fi

    # Initialize flags
    GNB_FLAG=false
    UE_FLAG=false
    NB_THREADS=10
    
    for arg in "$@"; do
        case $arg in
            --usrp=*)
                USRP_SERIAL="${arg#*=}"
                ;;
            --gnb)
                GNB_FLAG=true
                ;;
            --ue)
                UE_FLAG=true
                ;;
            --threads=*)
                NB_THREADS="${arg#*=}"
                if ! [[ "$NB_THREADS" =~ ^[0-9]+$ ]]; then
                    echo "Error: --threads must be a number."
                    usage
                fi
                ;;
            --help)
                usage
                ;;
            *)
                usage
                ;;
        esac
    done

    # Ensure that either --gnb or --ue is set, but not both
    if [ "$GNB_FLAG" == true ] && [ "$UE_FLAG" == true ]; then
        echo "Error: --gnb and --ue are exclusive parameters. Only one can be provided."
        usage
    fi

    if [ "$GNB_FLAG" == false ] && [ "$UE_FLAG" == false ]; then
        echo "Error: Either --gnb or --ue must be provided."
        usage
    fi
}


# get_interface_ip(destination_ip):source_ip
get_interface_ip() {
    local DEST_IP=$1
    iface=$(ip route get "$DEST_IP" | head -1 | grep -oP '(?<=dev )\S+')
    #ip route get "$DEST_IP" | grep -oP 'src \K[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+'
    ip -4 addr show dev $iface | grep -oP '(?<=inet\s)\d+(\.\d+){3}/\d+'
}

start_gnb_remote() {

    local REMOTE_HOST_USER=$1   # e.g. user@remote_host
    local REMOTE_PASS=$2        # remote password for non-interactive ssh/scp

    echo "Starting gNB with ORU_Support"

    
    export AMF_IP=$(cat ~/pos/params.5g.yaml | yq .GCN.multus.ips.amf.n2.ip)
    
    # Generate the gNB configuration file locally
    echo "Creating gnb configuration"
    envsubst < ~/xp/gnb.conf.env > ~/results/gnb.conf

    # Transfer the local configuration file to the remote host
    sshpass -p "$REMOTE_PASS" scp -o StrictHostKeyChecking=no ~/results/gnb.conf ${REMOTE_HOST_USER}:~/results/gnb.conf

    # Build the remote command.
    # Note: The command changes to the build directory, starts nr-softmodem in the background,
    # and then echoes the PID. The "-t" option in SSH forces a pseudo-terminal (required for some sudo cases).
#    REMOTE_CMD="cd ~/openairinterface5g/cmake_targets/ran_build/build && \
#./nr-softmodem -O ~/results/gnb.conf --thread-pool 11,12,13,14,15 -m 26 -t 26 > ~/results/gnb.stdout 2> ~/results/gnb.stderr & echo \$!"
    curl -X POST ORU.local:5000/start
    sleep 5

    curl -X POST UE.local:5000/cm/stop
    curl -X POST UE.local:5000/radio/off
    curl -X POST UE.local:5000/cm/start
    sleep 5
    curl -X POST UE.local:5000/radio/on
    sleep 120

    # Execute the command remotely (non-interactively)
#    GNB_PID=$(sshpass -p "$REMOTE_PASS" ssh -t ${REMOTE_HOST_USER} "$REMOTE_CMD")
#    echo "Remote gNB started with PID: $GNB_PID"
#    echo $GNB_PID > ~/results/gnb.pid

    # Optionally, copy the remote stdout and stderr files back to your local machine
    sshpass -p "$REMOTE_PASS" scp -o StrictHostKeyChecking=no ${REMOTE_HOST_USER}:~/gnb.stdout ~/results/gnb.stdout
    sshpass -p "$REMOTE_PASS" scp -o StrictHostKeyChecking=no ${REMOTE_HOST_USER}:~/gnb.stderr ~/results/gnb.stderr

    curl -X POST ORU.local:5000/stop
    curl -X POST UE.local:5000/cm/stop
    curl -X POST UE.local:5000/radio/off
}


# Main function to control the flow
main() {
    parse_args "$@"

    echo "172.25.0.2 ORU.local" | sudo tee -a /etc/hosts > /dev/null
    echo "10.64.45.35 UE.local" | sudo tee -a /etc/hosts > /dev/null


    apt install sshpass
    wget -qO /usr/local/bin/yq https://github.com/mikefarah/yq/releases/latest/download/yq_linux_amd64
    chmod a+x /usr/local/bin/yq

    mkdir -p ~/results

    if [ "$GNB_FLAG" == true ]; then
        start_gnb_remote root@172.25.0.2 1234
    fi
}

# Call the main function
main "$@"
